# Author: Kero van Gelder
# Copyright: Kero van Gelder, the GNU Lesser General Public License applies
# First version: 17 Jul 2001

require 'agenda/time'

module Agenda
=begin
== Time Interval
On this we can perform various calculations, of which the most important one is
overlap?(anInterval). Unfortunately Comparable can not be used, since intervals
only have a partial ordering.
=end
  class TimeInterval
    attr_accessor :time		# Ruby Time (Mon Jul 17 14:32:27 CET 2001)
    attr_accessor :duration	# seconds (OK, minutes would /probably/ do, but
				# seconds are an SI unit, minutes are not :)
    def initialize(t=Time.now(), dur=0)
      @time = t
      @duration = dur
    end

    def contains?(time)
      return (@time <= time and @time+@duration-1 >= time)
    end

    def overlap?(other)
      return false  if duration == AllDay and time + Day - 1 < other.time
      return false  if other.duration == AllDay and other.time + other.duration - 1 < time
      return false  if duration != AllDay and time + duration - 1 < other.time
      return false  if other.duration != AllDay and other.time + other.duration - 1 < time
      return true
    end

    # yet we allow for sorting, which is useful e.g. when duration is the same
    def <=>(other)
      time <=> other.time
    end
    include Comparable

    def ===(other)
      @time === other.time and dur == other.dur
    end
  end

  AllDay = -8

=begin
== Single Appointment
Like the ones you write down in your paper version.
This single apointment is based on seconds since 1970, which pinpoints it
uniquely and allows you to take your agenda into another timezone and see your
appointments move (as they actually do!).

What s*cks, is that I need dates before 1970, of course (birthdates, mainly)
=end
  class Appointment < TimeInterval
    attr_accessor :subject      # One line description (String)
    attr_accessor :description  # Full description (String)
    attr_accessor :location
    # advance time of alarm,
    # multiple categories,
    # email which originated the appointment.
    attr_accessor :repetition
    def repeats() 0 end
    def repeats=(newvalue) end
    # def rep_dur=(newvalue) end
    def appointment() self end

    alias TimeInterval_init initialize
    def initialize(t=Time.now(), dur=30*60, str="")
      raise "class abstracted"  if type() === "Appointment"
      TimeInterval_init(t, dur)
      @subject = str
      @description = str
      @repetition = []
    end

=begin
=== Matching Appointments
Will be based on days, instead of TimeInterval. This is to circumvent Daylight
Saving Time troubles, which are timezone related.
=end
    def match(interval)
      if @repetition.empty?
	overlap?(interval) ? [self] : []
      else
	@repetition.collect { |rep|
	  rep.match(interval)
	}.flatten
      end
    end

    include XmlElt
    def Appointment::xmlName() "appt" end
    def addAttribute(attrib, value=nil)
      case attrib
      when "time"
	@time = Time.at(value.to_i)
      when "dur"
	@duration = (value == "allday") ? AllDay : value.to_i
      when "rep"
	case value
	when "86400"
	  @repetition[0] = Repetition.new(self, "86400")
	when "604800"
	  @repetition[0] = Repetition.new(self, "W")
	when "1209600"
	  @repetition[0] = Repetition.new(self, "W2")
	end
 	#@repeats = value.to_i == 0 ? Never : value.to_i
      when "repdur"
	#@rep_dur = value.to_i
      when "loc"
	@location = value
      when "sub"
	@subject = value
      when "descr"
	@description = value.gsub(/\\n/, "\n")
      end
    end
    def addChild(child)
      if child.instance_of? Deviation  # from 0.7 format
	@repetition[0].addChild(child)
      elsif child.instance_of? Repetition
	@repetition << child
	child.appointment = self
      else
	raise "Author probably made a mistake in conversion of database format"
      end
    end
    def dump()
      result = %Q(<appt time="#{@time.to_i}" dur="#{(@duration == AllDay) ? 'allday' : @duration.to_i}" loc="#{@location}" sub="#{@subject}" descr=#{@description.inspect}>)
      if @repetition.empty?
	result[-1,0] = "/"
      else
	@repetition.each { |rep|
	  result << rep.dump()
        }
        result << "</appt>"
      end
      result
    end
  end
end
